/// \file ipcore_foc.c
/// \brief IP core "FOC"
#include <stdint.h>

#include "xil_printf.h"
#include "xtime_l.h"

#include "xparameters.h"

#include "foc.h"
#include "mytime.h"
#include "ipcore_data_capture.h"

#define	FOC_PARAMS	((volatile uint32_t*)(XPAR_AXI_REG32_0_S_AXI_BASEADDR + 0x40))
#define	FOC_STATUS	((volatile uint32_t*)(XPAR_AXI_REG32_0_S_AXI_BASEADDR + 0x80))

#define write_parameter(foc_index, foc_value)	do { FOC_PARAMS[foc_index] = foc_value; } while (0)
#define	read_parameter(foc_index)				(FOC_PARAMS[foc_index])

#define	read_status(foc_index)					(FOC_STATUS[foc_index])


/// FOC clock frequency, in Hz,
static const unsigned int	FOC_CLOCK_HZ = 100u * 1000u * 1000u;


// --------------------------------------------------------------------------
static const uint32_t const initial_params[] = {
		0x00000140u,
		0x00000000u,
		0xFFFFF000u,
		0x00000000u,
		0x00000000u,
		0x00001388u,
		0x00000000u,
		0x00000BB8u,
		0xFFFFFF38u,
		0xFFFFFFFBu,
		0x000002CFu,
		0xFFFFE300u,
		0xFFFFC100u,
		0x000046C8u,
		0x00003937u,
		0x00000000u
};

// --------------------------------------------------------------------------
/// Init the FOC IP core.
void init_foc(void)
{
	int i;

	write_parameter(0, 0);

    for (i=1; i<ARGS_SIZE; ++i) {
        write_parameter(i, initial_params[i]);
    }

    write_parameter(0, initial_params[0]);
}

// --------------------------------------------------------------------------
void foc_write_mode(const unsigned int mode)
{
	const uint32_t	control = read_parameter(CONTROL_REG);
	write_parameter(CONTROL_REG, (control & ~0x7u) | (mode & 0x7u));
}

// --------------------------------------------------------------------------
unsigned int foc_read_mode(void)
{
	const uint32_t	control = read_parameter(CONTROL_REG);
	return control & 0x7u;
}

// --------------------------------------------------------------------------
void foc_write_fixperiod(unsigned int delay)
{
	const uint32_t	control = read_parameter(CONTROL_REG);
	const uint32_t	mask = CONTROL_MAX_FIXPERIOD << CONTROL_BIT_FIXPERIOD;
	const uint32_t	new_control = (control & ~mask) | ((delay << CONTROL_BIT_FIXPERIOD) & mask);
	write_parameter(CONTROL_REG, new_control);
}

// --------------------------------------------------------------------------
static void foc_write_decimate(const unsigned int df)
{
	const uint32_t	control2 = read_parameter(CONTROL2_REG);
	const uint32_t	new_control2 = (control2 & ~CONTROL2_BITMASK_DECIMATION) | ((df << CONTROL2_BIT_DECIMATION) & CONTROL2_BITMASK_DECIMATION);
	write_parameter(CONTROL2_REG, new_control2);
}

// --------------------------------------------------------------------------
static void write_capture_source(const unsigned int capture_source)
{
	const uint32_t	control2 = read_parameter(CONTROL2_REG);
	const uint32_t	new_control2 = (control2 & ~0x7u) | (capture_source & 0x7u);
	write_parameter(CONTROL2_REG, new_control2);
}

// --------------------------------------------------------------------------
#define	MAX_ZERO_CROSS	300
unsigned int			zero_cross[MAX_ZERO_CROSS];

// --------------------------------------------------------------------------
void foc_init_motor(void)
{
	const unsigned int	fixed_period = 200;
	const float			clocks_per_rev = (float)(fixed_period) * (float)(CPR * CPR);
	const unsigned int	ms_to_sleep = (unsigned int)(2.0 * (1000.0 / FOC_CLOCK_HZ) * clocks_per_rev);
	unsigned int		zero_cross_count = 0;

	write_parameter(ANGLE_SH_REG, 0u);
	foc_write_fixperiod(fixed_period);
	msleep(100);

	// The forced rotation mode ensures that the encoder index is reset at least once.
	foc_write_mode(MODE_MANUAL_TORQUE_FLUX_FIXED_SPEED);

	while (is_data_capture_in_progress()) {
		msleep(10);
	}

	foc_write_decimate(0);
	write_capture_source(DATASOURCE_V_A_B_C);

	const uint64_t				t_end = time_us() + ms_to_sleep * 1000ull;
	// Count of Va samples.
	unsigned int				count_va = 0u;
	// Last Va sample.
	int16_t						last_va = 0;
	// Zero crossings.
	do {
		const unsigned int	current_angle = read_status(ANGLE_REG);

		start_data_capture();
		do {
			msleep(1);
		} while (is_data_capture_in_progress());

		const int16_t		this_va = data_capture_result(0);
		if (count_va>0 && last_va <= 0 && this_va>0 && zero_cross_count<MAX_ZERO_CROSS) {
			zero_cross[zero_cross_count] = current_angle;
			++zero_cross_count;
		}

		++count_va;
		last_va = this_va;
	} while (time_us() < t_end);


	if (zero_cross_count>0u) {
		// The offset of 150 works for the EDDP Kit.
		const unsigned int	offset = (3*CPR)/(10*PPR) -20;
		const unsigned int	new_shift = (zero_cross[zero_cross_count - 1u] + offset) % CPR;
		write_parameter(ANGLE_SH_REG, new_shift);
		xil_printf("New shift: %u\r\n", new_shift);
	}
	else {
		xil_printf("init_motor: No zero crosses found. Strange :( \r\n");
		// TODO: How to report the failure?
	}
	foc_write_mode(MODE_STOPPED);
}

// --------------------------------------------------------------------------
void foc_write_speed(int speed_rpm)
{
	write_parameter(RPM_SP_REG, speed_rpm);
}


// --------------------------------------------------------------------------
uint32_t read_foc_buttons(void)
{
	const uint32_t	r = read_status(4);
	return r;
}

// --------------------------------------------------------------------------
int read_foc_speed(void)
{
	const int	r = (int32_t)read_status(RPM_REG);
	return r;
}

// --------------------------------------------------------------------------
int read_foc_id(void)
{
	const int	r = (int32_t)read_status(ID_REG);
	return r;
}

// --------------------------------------------------------------------------
int read_foc_iq(void)
{
	const int	r = (int32_t)read_status(IQ_REG);
	return r;
}

// --------------------------------------------------------------------------
void print_foc_parameters(void)
{
	xil_printf("FOC parameters:\r\n");
	for (int i=0; i<ARGS_SIZE; ++i) {
		xil_printf("%02d: 0x%08X\r\n", i, read_parameter(i));
	}
}
