/// \file MotorTest_main.c
/// \brief

#include <math.h>		// sqrt
#include <stdio.h>
#include <stdint.h>		// uint32_t
#include <stdbool.h>	// uint8_t

#include "xil_printf.h"

#include "platform.h"
#include "foc.h"
#include "ipcore_data_capture.h"
#include "ipcore_foc.h"
#include "mytime.h"


/// Button debounce time.
#define	DEBOUNCE_MS		20u

/// Motor stopping time, in ms.
#define	STOP_TIME_MS	100u

/// Motor speed, in RPM.
#define	MOTOR_SPEED_RPM	3000

/// Report period, in ms.
#define	REPORT_PERIOD_MS	1000u

/// Current ADC to amperes
#define	ADC2A				0.00039


static uint64_t	btn1_uptime = 0;
static uint64_t	btn2_uptime = 0;
static uint64_t	btn3_uptime = 0;

static bool		init_motor_called = false;

// --------------------------------------------------------------------------
static void init_motor_if_necessary(void)
{
	if (!init_motor_called) {
		xil_printf("Started init motor...\r\n");
		foc_init_motor();
		init_motor_called = true;
	}
}

// --------------------------------------------------------------------------
static bool button_pressed(unsigned int bv, uint64_t* counter, const uint64_t now_ms)
{
	if (bv) {
		*counter = now_ms;
		return true;
	}
	else {
		const uint64_t	cnt = *counter;
		return cnt>0u && (now_ms - cnt) < DEBOUNCE_MS;
	}
}

// --------------------------------------------------------------------------
int main(int argc, char* argv[])
{
    init_platform();

    msleep(100);
    init_data_capture();
    init_foc();

    xil_printf("FOC demo program\r\n");

	uint64_t	now_ms = time_ms();
    uint64_t	stop_time_ms = now_ms;
    uint64_t	last_report_time_ms = now_ms;

	int			sum_speed = 0;
	double		sum_i = 0;
	int			count = 0;
	uint32_t	buttons;
	int			id;
	int			iq;
	char		xbuf[200];

    for (;;) {
    	buttons = read_foc_buttons();
    	now_ms = time_ms();

    	// Any buttons pressed?
    	if (button_pressed(buttons & BTN1, &btn1_uptime, now_ms)) {
    		init_motor_if_necessary();
    		foc_write_speed(-MOTOR_SPEED_RPM);
    		foc_write_mode(MODE_SPEED);
    		stop_time_ms = now_ms + STOP_TIME_MS;
    	}
    	else if (button_pressed(buttons & BTN2, &btn2_uptime, now_ms)) {
    		init_motor_if_necessary();
    		foc_write_speed(MOTOR_SPEED_RPM);
    		foc_write_mode(MODE_SPEED);
    		stop_time_ms = now_ms + STOP_TIME_MS;
    	}
    	else if (button_pressed(buttons & BTN3, &btn3_uptime, now_ms)) {
    		foc_write_mode(MODE_STOPPED);
    		xil_printf("Started init motor...\r\n");
    		foc_init_motor();
    		init_motor_called = true;
    	}
    	else {
    		if (now_ms > stop_time_ms) {
    			foc_write_mode(MODE_STOPPED);
    		}
    		else {
    			foc_write_speed(0);
    		}
    	}

    	// Status update.
    	sum_speed += read_foc_speed();
    	id = read_foc_id();
    	iq = read_foc_iq();
    	sum_i += sqrt(id*id + iq*iq);
    	++count;
    	msleep(2);

    	// Report stats.
		const unsigned int	foc_mode = foc_read_mode();
    	const unsigned int	report_period_ms = foc_mode == MODE_STOPPED ? (60u * REPORT_PERIOD_MS) : REPORT_PERIOD_MS;
    	const uint64_t		report_time_ms = last_report_time_ms + report_period_ms;
    	if (now_ms > report_time_ms && count>0) {
    		const double		speed_rpm = sum_speed / count;
    		const double		current = ADC2A * (sum_i / count);

    		sprintf(xbuf, "%s, %3.1f RPM, stator current %5.3fA\r\n", foc_mode==MODE_STOPPED ? "Stopped" : "Running", speed_rpm, current);
    		xil_printf("%s", xbuf);

    		last_report_time_ms = now_ms;
    		count = 0;
    		sum_speed = 0;
    		sum_i = 0;
    	}
    }

    cleanup_platform();
    return 0;
}
